#include "display.h"

#include <iostream>

#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkmm/messagedialog.h>

extern "C" G_MODULE_EXPORT
GtkWidget* create_Display(char*, char*, char*, int, int)
{
	Gtk::Widget *const widget = new Display();
	widget->show();
	return Gtk::manage(widget)->gobj();
}

Display::Display(void) : DrawingArea()
{
	std::cout << "Constructor called.";
	std::cout.flush();
	Glib::RefPtr<Gdk::GL::Config> glconfig;
	glconfig = Gdk::GL::Config::create(Gdk::GL::MODE_RGB | Gdk::GL::MODE_DOUBLE);
	
	if(!glconfig)
	{
		Gtk::MessageDialog e("Couldn't find OpenGL visual.\nPlease ensure that OpenGL drivers are installed correctly.", Gtk::MESSAGE_ERROR);
		e.run();
	}

	if(!set_gl_capability(glconfig) || !is_gl_capable())
	{
		Gtk::MessageDialog e("Couldn't set OpenGL capability.\nPlease ensure that OpenGL drivers are installed correctly.", Gtk::MESSAGE_ERROR);
		e.run();
	}
}

void Display::on_realize(void)
{
	std::cout << "\non_realize() called.";
	std::cout.flush();
	Gtk::DrawingArea::on_realize();
	
	Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();
	
	glwindow->gl_begin(get_gl_context());
	glClearColor(1.0, 0.0, 0.0, 1.0);
	glMatrixMode(GL_PROJECTION);
	glOrtho(-0.5, 0.5, -0.5, 0.5, -0.1, 0.1);
	glColor3f(1.0f, 0.0f, 0.0f);
	glDisable(GL_DEPTH_TEST);	
	glwindow->gl_end();
}

bool Display::on_configure_event(GdkEventConfigure *ev)
{
	Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();
	if(!glwindow) return(false);

	glwindow->gl_begin(get_gl_context());
	glViewport(0, 0, GLsizei(get_width()), GLsizei(get_height()));

	glwindow->gl_end();
	return(true);
}

bool Display::on_expose_event(GdkEventExpose *ev)
{
	Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();
	if(!glwindow) return(false);

	glwindow->gl_begin(get_gl_context());

	glClear(GL_COLOR_BUFFER_BIT);

	glwindow->swap_buffers();

	// Error detection.
	GLenum error = glGetError();
	if(error != GL_NO_ERROR)
	{
		std::cerr << "\n OpenGL error: " << gluErrorString(error);
		std::cerr.flush();
	}

	glwindow->gl_end();

	return(true);
}
